@@ -6,7 +6,7 @@
 #i - list of indexes for individual likelihood (used to subset weights and r22)
 #repweightVar  - the replicate weight for replicate variance estimation
 #inside - TRUE if inside mix, otherwise linear and inside=FALSE
-fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE, fast=TRUE) {
+fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE, fast=TRUE, verbose=FALSE) {
   if(length(wv) > 1) {
     stop("Must specify NULL weight or exactly one.")
   }
@@ -33,7 +33,10 @@ fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE,
   rr1m <- (1/(.Machine$double.eps)^0.5)/apply(rr1p, 2, max) 
   rr1q <- t(t(rr1p) * rr1m) 
   insd <- inside
+  Xnodes <- t(matrix(nodes, nrow=length(nodes), ncol=nrow(X_)))
 
+  prev_gr <- rep(Inf, ncol(X_))
+  max_x_gr <- ncol(X_) * 1e-5
   function(par, returnPosterior=FALSE, gr=FALSE, hess=FALSE) {
     # break up par into beta and residual components
     B <- par[1:K]
@@ -48,7 +51,7 @@ fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE,
       # form prediction
       XB <- as.vector(X_ %*% B)
       # residual, per indivudal (outer sapply), per node (inner sapply)
-      nodes.minus.XB <- t(t(matrix(nodes, nrow=length(nodes), ncol=length(XB))) - XB)
+      nodes.minus.XB <- t(Xnodes - XB)
       # likelihood of normal distribution for residuals nodes.minus.XB
       rr2 <- rr1p * ((1/(sqrt(2 * pi * s2))) * exp(-((nodes.minus.XB)^2/(2 * s2))))
       # final parameter done numerically
@@ -59,11 +62,18 @@ fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE,
       # likelihood of normal distribution for residuals nodes.minus.XB
       rr2_ <- rr1p * ((1/(sqrt(2 * pi * s_^2))) * exp(-((nodes.minus.XB)^2/(2 * s_^2))))
       trr2mxb <- t(rr2 * nodes.minus.XB)
-	    colSums_rr2 <- colSums(rr2)
+      colSums_rr2 <- colSums(rr2)
       
       # Hess loop in rcpp
-	    H <- calcHess(K, rr2, rr2_, trr2mxb, X_, nodes.minus.XB, w, s2, s_)
-	    
+
+      if(fast) {
+        H <- calcHess(K, rr2, rr2_, trr2mxb, X_, nodes.minus.XB, w, s2, s_)
+        if(any(H %in% c(NA, NaN, Inf, -Inf))) {
+          H <- slowHess(K, rr2, rr2_, trr2mxb, X_, nodes.minus.XB, w, s2, s_)
+        }
+      } else { 
+        H <- slowHess(K, rr2, rr2_, trr2mxb, X_, nodes.minus.XB, w, s2, s_)
+      }
       # now variance of variance term
       gr0 <- -2*sum(w * colSums( rr2 * (-1/s + 1*(nodes.minus.XB)^2/s^3))/colSums_rr2)
       # break up par into beta and residual components
@@ -81,27 +91,25 @@ fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE,
       return(H)
     }
     if(gr) {
-      # form prediction
       XB <- as.vector(X_ %*% B)
       # residual, per indivudal (outer sapply), per node (inner sapply)
-      nodes.minus.XB <- t(t(matrix(nodes, nrow=length(nodes), ncol=length(XB))) - XB)
+      nodes.minus.XB <- t(Xnodes - XB)
       # likelihood of normal distribution for residuals nodes.minus.XB
       rr2 <- rr1p * ((1/(sqrt(2 * pi * s2))) * exp(-((nodes.minus.XB)^2/(2 * s2))))
       gr_res <- rep(0, length(par))
       trr2mxb <- t(rr2 * nodes.minus.XB)
       denom <- colSums(rr2)
-      for(i in 1:K) {
-        if(fast) {
-          gr_res[i] <-  grSum2(w, trr2mxb, X_, i, s2, denom)
-        } else {
-          gr_res[i] <- -2 * sum(w * rowSums( trr2mxb * X_[,i] / s2)/denom)
-        }
+      rs <- w * rowSums( trr2mxb / s2)/denom
+      for(i in 1:K){
+        gr_res[i] <- -2 * sum( X_[,i] * rs)
+        # works like: gr_res[i] <- -2 * sum(w * rowSums( trr2mxb * X_[,i] / s2)/denom)
       }
       gr_res[K+1] <- -2* sum(w * colSums( rr2 * (-1/s + 1*(nodes.minus.XB)^2/s^3))/denom)
       if(par[(K+1)] < 1) {
         gr_res[K+1] <- gr_res[K+1] * 0.5 * exp(0.5*(par[(K+1)]-1)) 
       }
       gr_res[is.na(gr_res)] <- 0
+      prev_gr <<- gr_res
       return(gr_res) 
     }
     # form prediction
@@ -121,7 +129,8 @@ fn.regression <- function(X_, i=NULL, wv=NULL, rr1, stuDat, nodes, inside=TRUE,
       }
       return(data.frame(id=rownames(X_), mu=mu, sd=sd, stringsAsFactors=FALSE))
     }
-    return( -2*( sum(w * (log(cs) - log(rr1m)) ) + sum(w) * log(dTheta) ) )
+    res <- -2*( sum(w * (log(cs) - log(rr1m)) ) + sum(w) * log(dTheta) )
+    return( res )
   }
 }
 
@@ -460,3 +469,26 @@ grmLikelihood <- function (theta, d, score, a, D=1.7) {
   }
   pr
 }
+
+slowHess <- function(K, rr2, rr2_, trr2mxb, X_, nodes.minus.XB, w, s2, s_) {
+  colSums_rr2_ <- colSums(rr2_)
+  H <- matrix(0, nrow=K+1, ncol=K+1)
+  denom <- colSums(rr2)
+  denom2 <- denom^2
+  denomPrimeList <- list()
+  for(i in 1:K) {
+    fi <- t( trr2mxb * X_[,i]/s2)
+    num <- colSums(fi)
+    for(j in i:K) {
+      fj <- t( trr2mxb * X_[,j]/s2)
+      numPrime <- colSums(t(t(rr2) * X_[,i] * X_[,j] / s2) - t( t(fj * nodes.minus.XB) * X_[,i])/s2)
+      denomPrime <- colSums(fj)
+      H[j,i] <- H[i,j] <- 2*sum(w*(numPrime * denom + num * denomPrime) / denom2)
+    }
+    # do numerical cross partial of variance term, taking care of the possibility of exp(x-1) transform
+    gr0 <- -2 * sum(w * num/denom)
+    gr_ <- -2 * sum(w * colSums( t( t((rr2_ * nodes.minus.XB)) * X_[,i]/s_^2))/colSums_rr2_)
+    H[K+1,i] <- H[i, K+1] <- (gr_ - gr0)/1e-6
+  }
+  return(H)
+}
